/* ***** BEGIN LICENSE BLOCK *****
* Version: MPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Mozilla Public License Version
* 1.1 (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is by Greg Yardley
*
* The Initial Developer of the Original Code is
*   Greg Yardley
* Portions created by the Initial Developer are Copyright (C) 2007
* the Initial Developer. All Rights Reserved.
*
* Contributor(s):
*   Trevor Hobson
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the MPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the MPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

var objection = {
	nsIFilePicker: Components.interfaces.nsIFilePicker,
	nsILocalFile: new Components.Constructor("@mozilla.org/file/local;1", "nsILocalFile"),
	preferences: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.objection."),
	prefPlatform: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.objection.platform."),
//	nsIDOMSerializer: Components.classes["@mozilla.org/xmlextras/xmlserializer;1"].createInstance(Components.interfaces.nsIDOMSerializer), // For debugging
	nsISolRequest: Components.classes["@objection/solrequest;1"].createInstance(Components.interfaces.nsISolRequest),
	stringBundle: '',
	treeObject: null,
	treeObjectDetail: null,
	treeSettingThird: null,
	flashAppDir: null,
	flashObjectDir: null,
	flashSettingDir: null,
	flashSettingFile: null,
	xmlSettingFile: null,
	xmlSettingFileData: null,
	arrayObject: [],
	xmlObjectDetail: null,
	eleObjectDetail: [],
	arraySettingThird: [],
	countObject: 0,
	countSettingThird: 0,

	eleGlobalCamera: null,
	eleGlobalSpace: null,
	eleGlobalThird: null,
	eleGlobalCache: null,
	eleGlobalSecurity: null,
	eleGlobalUpdateDisabled: null,
	eleGlobalUpdateInterval: null,
	eleGlobalVideoAcceleration: null,

	sortOrderObject: ['site', 'object'],
	sortOrderThird: ['name'],

	locGlobal: 'Global',
	locNever: 'Never',
	locNone: 'None',
	locUnlimited: 'Unlimited',
	locAlwaysAsk: 'Always Ask',
	locAlwaysAllow: 'Always Allow',
	locAlwaysDeny: 'Always Deny',

	thisPlatform: '',

	debugLevel: 10,

	viewObject:
	{
		rowCount: 0,
		setTree: function(treeBox) {this.treeBox = treeBox;},
		getImageSrc: function(row, column) {},
		getProgressMode: function(row, column) {},
		getCellValue: function(row, column) {},
		getCellText: function(row, column) {
			var rv="";
			try
			{
				if (objection.arrayObject[row].isSite)
				{
					if (!objection.arrayObject[row].isRead)
					{
						objection.getSiteSettingDetail(row);
					}
					if (column.id=="colName")
					{
						rv = objection.arrayObject[row].site;
					}
					else if (column.id=="colUsed")
					{
						var usedSiteB = objection.arrayObject[row].used;
						var usedSiteK = usedSiteB / 1024;
						var usedSiteM = usedSiteK / 1024;
						if (usedSiteB < 1024)
						{
							rv = usedSiteB + " bytes";
						}
						else if (usedSiteK < 1024)
						{
							rv = usedSiteK.toFixed(1) + " KB";
						}
						else
						{
							rv = usedSiteM.toFixed(1) + " MB";
						}
					}
					else if (column.id=="colLimit")
					{
						var limitSite = objection.arrayObject[row].limit;
						if (limitSite == -1)
						{
							rv = objection.locNever;
						}
						else if (limitSite === 0)
						{
							rv = objection.locNone;
						}
						else if (limitSite == 10)
						{
							rv = "10 KB";
						}
						else if (limitSite == 100)
						{
							rv = "100 KB";
						}
						else if (limitSite == 1024)
						{
							rv = "1 MB";
						}
						else if (limitSite == 10240)
						{
							rv = "10 MB";
						}
						else if (limitSite == -2)
						{
							rv = objection.locUnlimited;
						}
						else
						{
							rv = objection.locGlobal;
						}
					}
					else if (column.id=="colPrivacy")
					{
						var privacySite = objection.arrayObject[row].privacy;
						if (privacySite == 1)
						{
							rv = objection.locAlwaysAsk;
						}
						else if (privacySite == 2)
						{
							rv = objection.locAlwaysAllow;
						}
						else if (privacySite == 3)
						{
							rv = objection.locAlwaysDeny;
						}
						else
						{
							rv = objection.locGlobal;
						}
					}
				}
				else
				{
					if (column.id=="colName")
					{
						rv = objection.arrayObject[row].object;
					}
					else if (column.id=="colUsed")
					{
						rv = objection.arrayObject[row].size + " bytes";
						var sizeSiteB = objection.arrayObject[row].size;
						var sizeSiteK = sizeSiteB / 1024;
						var sizeSiteM = sizeSiteK / 1024;
						if (sizeSiteB < 1024)
						{
							rv = sizeSiteB + " bytes";
						}
						else if (sizeSiteK < 1024)
						{
							rv = sizeSiteK.toFixed(1) + " KB";
						}
						else
						{
							rv = sizeSiteM.toFixed(1) + " MB";
						}
					}
					else if (column.id=="colLimit")
					{
						rv = "";
					}
					else if (column.id=="colPrivacy")
					{
						rv = "";
					}
				}
			}
			catch(err)
			{
				objection.jsdump('OBJECTION-Content viewObject:error: row' + row + ' err:' + err, 1);
			}
			return rv;
		},
		getParentIndex: function(row) {
			var rv = -1;
			try
			{
				if (objection.arrayObject[row].level > 0)
				{
					for (var t=row-1; t>=0; t--)
					{
						if (objection.arrayObject[t].level < objection.arrayObject[row].level)
						{
							rv = t;
						}
					}
				}
			} catch(err){}
			return rv;
		},
		getLevel: function(row) {
			var rv = 0;
			try
			{
				rv = objection.arrayObject[row].level;
			} catch(err){}
			return rv;
		},
		isSeparator: function(index) {return false;},
		isSorted: function() {return false;},
		isContainer: function(row) {return false;}, // Say false because we don't want the twistie
		isContainerOpen: function(row) {return true;},
		isContainerEmpty: function(row) {return false;},
		isEditable: function(row, column) {return false;},
		hasNextSibling: function(row, after) {
			var rv = false;
			try
			{
				var thisLevel = objection.arrayObject[row].level;
				for (var t = row+1; t < objection.arrayObject.length; t++)
				{
					var nextLevel = objection.arrayObject[t].level;
					if (nextLevel == thisLevel)
					{
						rv = true;
						break;
					}
					else if (nextLevel < thisLevel)
					{
						rv = false;
						break;
					}
				}
			} catch(err){}
			return rv;
		},
		toggleOpenState: function(row) {},
		cycleHeader: function(aColId, aElt) {},
		selectionChanged: function() {},
		cycleCell: function(row, column) {},
		performAction: function(action) {},
		performActionOnCell: function(action, row, column) {},
		getRowProperties: function(row, props) {},
		getColumnProperties: function(column, props) {},
		getCellProperties: function(row, column, props) {}
	},

	viewObjectDetail:
	{
		rowCount: 0,
		setTree: function(treeBox) {this.treeBox = treeBox;},
		getImageSrc: function(row, column) {},
		getProgressMode: function(row, column) {},
		getCellValue: function(row, column) {},
		getCellText: function(row, column) {
			var rv="";
			try
			{
				if (column.id=="colName")
				{
					rv = objection.eleObjectDetail[row].getAttribute('name');
				}
				else if (column.id=="colType")
				{
					rv = objection.eleObjectDetail[row].getAttribute('type');
				}
				else if (column.id=="colValue")
				{
					var rowValue = objection.eleObjectDetail[row].firstChild.nodeValue;
					if (objection.eleObjectDetail[row].getAttribute('type') == "pointer")
					{
						var rowTarget = objection.evaluateXPath(objection.xmlObjectDetail, "//data/element[@target='" + rowValue + "']");
						if (rowTarget.length > 0)
						{
							rv = '[' + rowValue + '] ' + rowTarget[0].getAttribute('name') + ' (' + rowTarget[0].getAttribute('type') + ')';
						}
						else
						{
							rv = '[' + rowValue + '] err';
						}
					}
					else
					{
						rv = rowValue;
					}
				}
			} catch(err){}
			return rv;
		},
		getParentIndex: function(row) {
			var rv = -1;
			try
			{
				var rowParent = parseInt(objection.eleObjectDetail[row].getAttribute('parent'));
				rv = (rowParent != 0) ? rowParent : -1;
			} catch(err){}
			return rv;
		},
		getLevel: function(row) {
			var rv = 0;
			try
			{
				rv = parseInt(objection.eleObjectDetail[row].getAttribute('level'));
			} catch(err){}
			return rv;
		},
		isSeparator: function(index) {return false;},
		isSorted: function() {return false;},
		isContainer: function(row) {return false;}, // Say false because we don't want the twistie
		isContainerOpen: function(row) {return true;},
		isContainerEmpty: function(row) {return false;},
		isEditable: function(row, column) {return false;},
		hasNextSibling: function(row, after) {
			var rv = false;
			try
			{
				var rowParent = parseInt(objection.eleObjectDetail[row].getAttribute('parent'));
				var rowSibling = objection.evaluateXPath(objection.xmlObjectDetail, "//data/element[@parent='" + rowParent + "' and @number>'" + row + "'][1]");
				rv = (rowSibling.length != 0) ? true : false;
			} catch(err){}
			return rv;
		},
		toggleOpenState: function(row) {},
		cycleHeader: function(aColId, aElt) {},
		selectionChanged: function() {},
		cycleCell: function(row, column) {},
		performAction: function(action) {},
		performActionOnCell: function(action, row, column) {},
		getRowProperties: function(row, props) {},
		getColumnProperties: function(column, props) {},
		getCellProperties: function(row, column, props) {}
	},

	viewSettingThird:
	{
		rowCount: 0,
		setTree: function(treeBox) {this.treeBox = treeBox;},
		getImageSrc: function(row, column) {},
		getProgressMode: function(row, column) {},
		getCellValue: function(row, column) {},
		getCellText: function(row, column) {
			var rv="";
			if (column.id=="colSite")
			{
				rv = objection.arraySettingThird[row];
			}
			return rv;
		},
		isSeparator: function(index) {return false;},
		isSorted: function() {return false;},
		isContainer: function(index) {return false;},
		cycleHeader: function(aColId, aElt) {},
		getRowProperties: function(row, props) {},
		getColumnProperties: function(column, props) {},
		getCellProperties: function(row, column, props) {}
	},

	frameObject: function(number, site, object, used, size, limit, privacy, level, file, isSite, isRead)
	{
		this.number = number;
		this.site = site;
		this.object = object;
		this.used = used;
		this.size = size;
		this.limit = limit;
		this.privacy = privacy;
		this.level = level;
		this.file = file;
		this.isSite = isSite;
		this.isRead = isRead;
		return this;
	},

	init: function()
	{
		// Get the debug level
		// Levels are: 1-All alerts, 5-Failure alerts, 10-No alerts
		this.debugLevel = 10;
		try {this.debugLevel = this.preferences.getIntPref("debugLevel");}catch (err) {}

		// Reference string bundle set
		this.stringBundle = document.getElementById('objectionStrings');

		// Read string bundle to variables (not all)
		this.locGlobal = this.stringBundle.getString("global");
		this.locNever = this.stringBundle.getString("never");
		this.locNone = this.stringBundle.getString("none");
		this.locUnlimited = this.stringBundle.getString("unlimited");
		this.locAlwaysAsk = this.stringBundle.getString("alwaysAsk");
		this.locAlwaysAllow = this.stringBundle.getString("alwaysAllow");
		this.locAlwaysDeny = this.stringBundle.getString("alwaysDeny");

		// Reference the trees
		this.treeObject = document.getElementById('treeObject');
		this.treeObjectDetail = document.getElementById('treeObjectDetail');
		this.treeSettingThird = document.getElementById('treeSettingThird');

		// Init the File/Directory objects
		this.flashAppDir = new this.nsILocalFile();
		this.flashSettingDir = new this.nsILocalFile();
		this.flashSettingFile = new this.nsILocalFile();
		this.flashObjectDir = new this.nsILocalFile();

		// Get the platform for the Flash Data Directory
		this.thisPlatform = window.navigator.platform;
		this.thisPlatform = this.thisPlatform.replace(/[\.| ]/g,'_');

		// Get the Flash data directory
		var flashAppDirPref = '';
		if (this.prefPlatform.prefHasUserValue(this.thisPlatform) && typeof(this.prefPlatform.getCharPref(this.thisPlatform)) != 'undefined')
		{
			flashAppDirPref = this.prefPlatform.getCharPref(this.thisPlatform);
			if (flashAppDirPref !== '')
			{
				this.flashAppDir.initWithPath(flashAppDirPref);
				if (!(this.flashAppDir.exists() && this.flashAppDir.isDirectory()))
				{
					this.jsdump('OBJECTION-Content init:failed:Invalid Platform Directory', 1);
					this.flashAppDir = new this.nsILocalFile();
				}
			}
			else
			{
				this.jsdump('OBJECTION-Content init:failed:Invalid Platform Directory', 1);
				this.flashAppDir = new this.nsILocalFile();
			}
		}
		if (this.flashAppDir.path === '')
		{
			// Check the non platfrom specific preference (The old one)
			this.flashAppDir = new this.nsILocalFile();
			if (this.preferences.prefHasUserValue("flashAppDir") && typeof(this.preferences.getCharPref("flashAppDir")) != 'undefined')
			{
				flashAppDirPref = this.preferences.getCharPref("flashAppDir");
				this.flashAppDir.initWithPath(flashAppDirPref);
				if (this.flashAppDir.exists() && this.flashAppDir.isDirectory())
				{
					this.jsdump('OBJECTION-Content init:success:Using Old Pref', 1);
					this.prefPlatform.setCharPref(this.thisPlatform, flashAppDirPref);
				}
				else
				{
					this.flashAppDir = this.getFlashAppDir();  // Old preference may have been from a different platform, so try and find the one for this
				}
			}
			else
			{
				this.flashAppDir = this.getFlashAppDir();
			}
		}
		if (this.flashAppDir === null)
		{
			this.flashAppDir = new this.nsILocalFile();
		}
		// If there is not a valid Flash data directory then send the user to the Folder pane
		if (this.flashAppDir.path === '')
		{
			document.getElementById("flashAppDir").value = this.stringBundle.getString("dirNotFound");
			document.getElementById("objectionPreferences").showPane(document.getElementById("paneFolder"));
			return;
		}
		document.getElementById("flashAppDir").value = this.flashAppDir.path;

		// Set the options (Folder) pane
		this.onAutomaticRemoveChange();

		// Reset arrays
		this.arrayObject = new Array();
		this.eleObjectDetail = new Array();
		this.arraySettingThird = new Array();

		// Reset counters
		this.countObject = 0;
		this.countObjectDetail = 0;
		this.countSettingThird = 0;

		// Rest setting file elements
		this.eleGlobalCamera = null;
		this.eleGlobalSpace = null;
		this.eleGlobalThird = null;
		this.eleGlobalCache = null;
		this.eleGlobalSecurity = null;
		this.eleGlobalUpdateDisabled = null;
		this.eleGlobalUpdateInterval = null;
		this.eleGlobalVideoAcceleration = null;

		// Init the Object Detail Tree
		this.viewObjectDetail.rowCount = 0;
		this.treeObjectDetail.view = this.viewObjectDetail;

		// Run the main function
		this.main();
	},

	main: function()
	{
		var hasFlashSetting = this.getFlashSetting();
		if (this.flashSettingDir !== null)
		{
			this.getSiteSetting();
		}
		var hasObjectDir = this.getObjectDir();
		if (hasFlashSetting)
		{
			// Show flash settings
			this.showSettingFlash();

			// Sort third (party) array
			this.sortArrayAscending(this.arraySettingThird, this.sortOrderThird);

			// Set third (party) tree
			this.viewSettingThird.rowCount = this.arraySettingThird.length;
			this.treeSettingThird.view = this.viewSettingThird;
			this.treeSettingThird.treeBoxObject.invalidate();
		}
		if (hasObjectDir)
		{
			this.getObject();

			// Sort object array
			this.sortArrayAscending(this.arrayObject, this.sortOrderObject);

			// Set object tree
			this.viewObject.rowCount = this.arrayObject.length;
			this.treeObject.view = this.viewObject;
			this.treeObject.treeBoxObject.invalidate();
		}
	},

	// Get Flash player settings
	getFlashSetting: function()
	{
		var returnSet = false;
		if (this.flashAppDir.path === '')
		{
			this.flashSettingDir = null;
			this.flashSettingFile = null;
			return returnSet;
		}
		this.flashSettingDir.initWithFile(this.flashAppDir);
		this.flashSettingDir.appendRelativePath('macromedia.com');
		this.flashSettingDir.appendRelativePath('support');
		this.flashSettingDir.appendRelativePath('flashplayer');
		this.flashSettingDir.appendRelativePath('sys');
		if (this.flashSettingDir.exists() && this.flashSettingDir.isDirectory())
		{
			this.flashSettingFile.initWithFile(this.flashSettingDir);
			this.flashSettingFile.append('settings.sol');
			if (this.flashSettingFile.exists() && this.flashSettingFile.isFile())
			{
				this.xmlSettingFile = this.nsISolRequest.solRead(this.flashSettingFile);
				if (this.xmlSettingFile !== null)
				{
					returnSet = true;
					var listSettingFileData = this.evaluateXPath(this.xmlSettingFile, "//data");
					this.xmlSettingFileData = (listSettingFileData.length > 0) ? listSettingFileData[0] : null;
					this.eleGlobalCamera = this.setEle(this.xmlSettingFile, 'defaultalways');
					this.eleGlobalSpace = this.setEle(this.xmlSettingFile, 'defaultklimit');
					this.eleGlobalThird = this.setEle(this.xmlSettingFile, 'allowThirdPartyLSOAccess');
					this.eleGlobalCache = this.setEle(this.xmlSettingFile, 'secureCrossDomainCacheSize');
					this.eleGlobalSecurityAllow = this.setEle(this.xmlSettingFile, 'crossdomainAllow');
					this.eleGlobalSecurityAlways = this.setEle(this.xmlSettingFile, 'crossdomainAlways');
					this.eleGlobalUpdateDisabled = this.setEle(this.xmlSettingFile, 'autoUpdateDisabled');
					this.eleGlobalUpdateInterval = this.setEle(this.xmlSettingFile, 'autoUpdateInterval');
					this.eleGlobalVideoAcceleration = this.setEle(this.xmlSettingFile, 'safefullscreen');
					var eleTrustedPaths = this.setEle(this.xmlSettingFile, 'trustedPaths');
					var eleDomains = this.setEle(this.xmlSettingFile, 'domains');

					// Get the list of domains from the setting file
					if (eleDomains !== null)
					{
						var numDomains = eleDomains.getAttribute("number");
						var listDomains = this.evaluateXPath(this.xmlSettingFile, "//data/element[@parent='" + numDomains + "']");
						var flashSettingSite = '';
						for (var i=0; i < listDomains.length; i++)
						{
							flashSettingSite = (listDomains[i].getAttribute('name') != 'local') ? listDomains[i].getAttribute('name') : 'localhost';
							this.arrayObject[this.countObject] = new this.frameObject(this.countObject++, flashSettingSite, '', 0, -10, -10, -10, 0, null, true, false);
							// Remove the domain from the setting file (Domains will be added before saving)
							this.xmlSettingFile.firstChild.removeChild(listDomains[i]);
						}
						// Remove the domains element (It will be added before saving)
						this.xmlSettingFile.firstChild.removeChild(eleDomains);
					}
					// Get the list of trusted paths from the setting file
					if (eleTrustedPaths !== null)
					{
						var numTrustedPaths = eleTrustedPaths.getAttribute('number');
						var listTrustedPaths = this.evaluateXPath(this.xmlSettingFile, "//data/element[@parent='" + numTrustedPaths + "']");
						for (var i=0; i < listTrustedPaths.length; i++)
						{
							this.arraySettingThird[this.countSettingThird] = listTrustedPaths.getAttribute('name');
							this.countSettingThird++;
							// Remove the trusted path from the setting file (Trusted paths will be added before saving)
							this.xmlSettingFile.firstChild.removeChild(listTrustedPaths[i])
						}
						// Remove the trusted paths element (It will be added before saving)
						this.xmlSettingFile.firstChild.removeChild(eleTrustedPaths);
					}
				}
			}
			else
			{
				this.flashSettingFile = null;
			}
		}
		else
		{
			this.flashSettingDir = null;
			this.flashSettingFile = null;
		}
		return returnSet;
	},

	getSiteSetting: function()
	{
		try
		{
			var entries = this.flashSettingDir.directoryEntries;
		} catch(err)
		{
			return;
		}
		while(entries.hasMoreElements())
		{
			var entry = entries.getNext();
			entry.QueryInterface(Components.interfaces.nsIFile);
			if (entry.isDirectory())
			{
				var settingSite = entry.leafName;
				settingSite = settingSite.replace('#','');
				settingSite = (settingSite != 'local') ? settingSite : 'localhost';
				var isSet = false;
				entry.append('settings.sol');
				var settingFile = null;
				if (entry.exists() && entry.isFile())
				{
					settingFile = entry;
				}
				for (var j=0; j < this.arrayObject.length; j++)
				{
					if (this.arrayObject[j].site == settingSite)
					{

						this.arrayObject[j].file = settingFile;
						isSet = true;
						break;
					}
				}
				if (!isSet)
				{
					this.arrayObject[this.countObject] = new this.frameObject(this.countObject++, settingSite, '', 0, -1, -10, -10, 0, settingFile, true, false);
				}
			}
		}
	},

	getSiteSettingDetail: function(indexObject)
	{
		if (this.arrayObject[indexObject].isSite && !this.arrayObject[indexObject].isRead)
		{
			var settingFile = this.arrayObject[indexObject].file;
			if (settingFile !== null)
			{
				if (settingFile.exists() && settingFile.isFile())
				{
					var xmlSiteSetting = this.nsISolRequest.solRead(settingFile);
					var eleSiteSpace = this.setEle(xmlSiteSetting, 'klimit');
					var eleSiteSecurityAllow = this.setEle(xmlSiteSetting, 'allow');
					var eleSiteSecurityAlways = this.setEle(xmlSiteSetting, 'always');
					var siteSettingSpace = (eleSiteSpace !== null) ? eleSiteSpace.firstChild.nodeValue : '-10';
					var siteSettingSecurity = '-10';
					if (eleSiteSecurityAllow !== null && eleSiteSecurityAlways !== null)
					{
						var bolSiteSecurityAllow = this.toBoolean(eleSiteSecurityAllow.firstChild.nodeValue);
						var bolSiteSecurityAlways = this.toBoolean(eleSiteSecurityAlways.firstChild.nodeValue);
						siteSettingSecurity = (bolSiteSecurityAllow && bolSiteSecurityAlways) ? '2' : (bolSiteSecurityAlways) ? '3' : '1';
					}
					this.arrayObject[indexObject].limit = siteSettingSpace;
					this.arrayObject[indexObject].privacy = siteSettingSecurity;
				}
			}
			this.arrayObject[indexObject].isRead = true;
		}
	},

	getObjectDir: function()
	{
		var returnSet = false;
		if (this.flashAppDir.path === '')
		{
			this.flashObjectDir = null;
			return returnSet;
		}
		this.flashObjectDir.initWithFile(this.flashAppDir);
		this.flashObjectDir.appendRelativePath('#SharedObjects');
		if (this.flashObjectDir.exists() && this.flashObjectDir.isDirectory())
		{
			try
			{
				var entries = this.flashObjectDir.directoryEntries;
				while(entries.hasMoreElements())
				{
					var entry = entries.getNext();
					entry.QueryInterface(Components.interfaces.nsIFile);
					if (entry.isDirectory())
					{
						this.flashObjectDir.initWithFile(entry);
						returnSet = true;
						break;
					}
				}
			} catch(err) {}
		}
		return returnSet;
	},

	getObject: function()
	{
		try
		{
			var entries = this.flashObjectDir.directoryEntries;
		} catch(err)
		{
			return;
		}
		while(entries.hasMoreElements())
		{
			var entry = entries.getNext();
			entry.QueryInterface(Components.interfaces.nsIFile);
			if (entry.isDirectory())
			{
				var settingSite = entry.leafName;
				var settingSiteId = -1;
				for (var j=0; j < this.arrayObject.length; j++)
				{
					if (this.arrayObject[j].site == settingSite && this.arrayObject[j].isSite)
					{
						settingSiteId = j;
						break;
					}
				}
				if (settingSiteId < 0)
				{
					settingSiteId = this.countObject;
					this.arrayObject[this.countObject] = new this.frameObject(this.countObject++, settingSite, '', 0, -1, -10, -10, 0, null, true, false);
				}
				this.getObjectSite(settingSite, settingSiteId, entry);
			}
		}
	},

	getObjectSite: function(settingSite, settingSiteId, searchDir)
	{
		try
		{
			var entries = searchDir.directoryEntries;
		} catch(err)
		{
			return;
		}
		while(entries.hasMoreElements())
		{
			var entry = entries.getNext();
			entry.QueryInterface(Components.interfaces.nsIFile);
			if (entry.isFile())
			{
				var leaf = entry.leafName;
				var dotIndex  = leaf.lastIndexOf('.');
				var extension = (dotIndex >= 0) ? leaf.substring(dotIndex+1) : "";
				if ((extension == "sol")||(extension == "fso"))
				{
					this.arrayObject[this.countObject] = new this.frameObject(this.countObject++, settingSite, entry.leafName, -1, entry.fileSize, -10, -10, 1, entry, false, false);
					this.arrayObject[settingSiteId].used = this.arrayObject[settingSiteId].used + entry.fileSize;
				}
			}
			else if (entry.isDirectory())
			{
				this.getObjectSite(settingSite, settingSiteId, entry);
			}
		}
	},

	setEle: function(checkXml, itemName)
	{
		var returnElement = null;
		if (checkXml !== null)
		{
			findElement = this.evaluateXPath(checkXml, "//data/element[@name='" + itemName + "']");
			if (findElement.length !== 0)
			{
				returnElement = findElement[0];
			}
		}
		return returnElement;
	},

	treeObjectSelected: function()
	{
		var indexObject = this.treeObject.currentIndex;
		if (indexObject > -1)
		{
			if (this.arrayObject[indexObject].isSite)
			{
				document.getElementById("groupObjectDetail").setAttribute("collapsed", "true");
				document.getElementById("buttonObjectDeleteSelected").setAttribute("collapsed", "true");
				document.getElementById("groupSiteDetail").removeAttribute("collapsed");
				document.getElementById("buttonSiteDeleteSelected").removeAttribute("collapsed");
			}
			else
			{
				document.getElementById("groupSiteDetail").setAttribute("collapsed", "true");
				document.getElementById("buttonSiteDeleteSelected").setAttribute("collapsed", "true");
				document.getElementById("groupObjectDetail").removeAttribute("collapsed");
				document.getElementById("buttonObjectDeleteSelected").removeAttribute("collapsed");
			}
			this.showObjectDetail(indexObject);
		}
		else
		{
			document.getElementById("groupObjectDetail").setAttribute("collapsed", "true");
			document.getElementById("groupSiteDetail").setAttribute("collapsed", "true");
			document.getElementById("buttonObjectDeleteSelected").setAttribute("collapsed", "true");
			document.getElementById("buttonSiteDeleteSelected").setAttribute("collapsed", "true");
			this.hideObjectDetail();
		}
	},

	treeObjectKeyPress: function(e)
	{
		if (e.keyCode == 46)
		{
			this.deleteObjectSelected();
		}
	},

	treeSettingThirdSelected: function()
	{
		var indexObject = this.treeSettingThird.currentIndex;
		if (indexObject > -1)
		{
			document.getElementById("buttonDeleteThirdSelected").removeAttribute("disabled");
		}
		else
		{
			document.getElementById("buttonDeleteThirdSelected").setAttribute("disabled", "true");
		}
	},

	showObjectDetail: function(indexObject)
	{
		// Reset the tree
		this.treeObjectDetail.treeBoxObject.beginUpdateBatch();
		var oldLength = this.viewObjectDetail.rowCount;
		this.viewObjectDetail.rowCount = 0;
		this.treeObjectDetail.treeBoxObject.rowCountChanged(0, -oldLength);
		this.eleObjectDetail = new Array();
		if (!this.arrayObject[indexObject].isSite)
		{
			// Show sol file path
			var pathSolFull = this.arrayObject[indexObject].file.path;
			var pathSolShort = '...' + pathSolFull.substring(this.flashObjectDir.path.length);
			document.getElementById("textObjectDetailPath").value = pathSolShort;

			// If sol file still exists (may have been moved or deleted)
			if (this.arrayObject[indexObject].file.exists() && this.arrayObject[indexObject].file.isFile())
			{
				// Get sol file content
				this.xmlObjectDetail = this.nsISolRequest.solRead(this.arrayObject[indexObject].file);
				this.eleObjectDetail = this.evaluateXPath(this.xmlObjectDetail, "//data/element");

				// Show sol content in tree
				this.viewObjectDetail.rowCount = this.eleObjectDetail.length;
				this.treeObjectDetail.treeBoxObject.rowCountChanged(0, this.eleObjectDetail.length);
			}
		}
		else
		{
			// Show site Used space
			var siteUsed = 0;
			var siteUsedB = this.arrayObject[indexObject].used;
			var siteUsedK = siteUsedB / 1024;
			var siteUsedM = siteUsedK / 1024;
			if (siteUsedB < 1024)
			{
				siteUsed = siteUsedB + " bytes";
			}
			else if (siteUsedK < 1024)
			{
				siteUsed = siteUsedK.toFixed(1) + " KB";
			}
			else
			{
				siteUsed = siteUsedM.toFixed(1) + " MB";
			}
			document.getElementById("textSiteUsed").value = siteUsed;

			// Set delete site LSOs button state
			if (siteUsedB > 0)
			{
				document.getElementById("buttonSiteDeleteSelected").removeAttribute("disabled");
			}
			else
			{
				document.getElementById("buttonSiteDeleteSelected").setAttribute("disabled", "true");
			}

			// Show site Limit and Privacy
			var siteLimit = this.arrayObject[indexObject].limit;
			var sitePrivacy = this.arrayObject[indexObject].privacy;
			document.getElementById("settingSiteSpace").value = siteLimit;
			document.getElementById("settingSiteCamera").value = sitePrivacy;

			// Set delete settings button state
			if (this.arrayObject[indexObject].file !== null)
			{
				document.getElementById("buttonSiteDeleteSetting").removeAttribute("disabled");
			}
			else
			{
				document.getElementById("buttonSiteDeleteSetting").setAttribute("disabled", "true");
			}
		}
		// Show sol content in tree
		this.treeObjectDetail.treeBoxObject.endUpdateBatch();
	},

	hideObjectDetail: function()
	{
		// Reset detail elements and tree
		this.eleObjectDetail = new Array();
		this.viewObjectDetail.rowCount = 0;
		this.treeObjectDetail.view = this.viewObjectDetail;
		this.treeObjectDetail.treeBoxObject.invalidate();

		// Remove the path
		document.getElementById("textObjectDetailPath").value = '';
	},

	deleteSiteSelected: function()
	{
		var indexObject = this.treeObject.currentIndex;
		if (indexObject > -1)
		{
			if (this.arrayObject[indexObject].isSite)
			{
				if (confirm(this.stringBundle.getString("confirmSiteLsoRemove")))
				{
					var objectSite = this.arrayObject[indexObject].site;
					this.arrayObject[indexObject].used = 0;
					var deleteCount = 0;
					var treeBox = this.treeObject.treeBoxObject;
					treeBox.beginUpdateBatch();
					for (var j=this.arrayObject.length-1; j>-1; j--)
					{
						if (this.arrayObject[j].site == objectSite && !this.arrayObject[j].isSite)
						{
							// Delete the file
							this.deleteFile(this.arrayObject[j].file, false);
							deleteCount++;
							this.arrayObject.splice(j, 1);
						}
					}
					this.viewObject.rowCount -= deleteCount;
					treeBox.rowCountChanged(indexObject, -deleteCount);
					treeBox.view.selection.select(indexObject < this.arrayObject.length ? indexObject : this.arrayObject.length - 1);
					treeBox.endUpdateBatch();
					treeBox.ensureRowIsVisible(this.treeObject.currentIndex);

					// Delete the whole site directory
					this.deleteObjectSiteDir(objectSite);
				}
			}
		}
	},

	deleteObjectSelected: function()
	{
		var indexObject = this.treeObject.currentIndex;
		if (indexObject > -1)
		{
			if (!this.arrayObject[indexObject].isSite)
			{
				if (confirm(this.stringBundle.getString("confirmLsoRemove")))
				{
					// Delete the file
					this.deleteFile(this.arrayObject[indexObject].file, false);

					// Remove the file size from the site listing
					var objectSize = this.arrayObject[indexObject].size;
					var objectSite = this.arrayObject[indexObject].site;
					for (var j=0; j < this.arrayObject.length; j++)
					{
						if (this.arrayObject[j].site == objectSite && this.arrayObject[j].isSite)
						{
							this.arrayObject[j].used -= objectSize;
							if (this.arrayObject[j].used === 0)
							{
								// Delete the whole site directory
								this.deleteObjectSiteDir(this.arrayObject[j].site);
							}
							break;
						}
					}

					// Delete from tree and update
					this.deleteTreeSelectionSingle(this.treeObject, this.viewObject, this.arrayObject);
					this.treeObjectSelected();
				}
			}
		}
	},

	deleteObjectAll: function()
	{
		if (confirm(this.stringBundle.getString("confirmLsoRemoveAll")))
		{
			var treeBox = this.treeObject.treeBoxObject;
			treeBox.beginUpdateBatch();
			for (var j=this.arrayObject.length-1; j>-1; j--)
			{
				if (this.arrayObject[j].isSite)
				{
					this.arrayObject[j].used = 0;
					// If the site does not have a settings file
					if (this.arrayObject[j].file === null)
					{
						// Delete the site from the array (tree)
						this.arrayObject.splice(j, 1);
						this.viewObject.rowCount--;
						treeBox.rowCountChanged(j, -1);
					}
				}
				else
				{
					// Delete the object
					this.deleteFile(this.arrayObject[j].file, false);
					// Delete the object from the array (tree)
					this.arrayObject.splice(j, 1);
					this.viewObject.rowCount--;
					treeBox.rowCountChanged(j, -1);
				}
			}
			treeBox.view.selection.select(-1);
			treeBox.endUpdateBatch();
			treeBox.ensureRowIsVisible(this.treeObject.currentIndex);

			// Delete all site directories
			try
			{
				var entries = this.flashObjectDir.directoryEntries;
			} catch(err)
			{
				return;
			}
			while(entries.hasMoreElements())
			{
				var entry = entries.getNext();
				entry.QueryInterface(Components.interfaces.nsIFile);
				if (entry.isDirectory())
				{
					this.deleteFile(entry, true);
				}
			}
		}
	},

	deleteSiteSetting: function()
	{
		var indexObject = this.treeObject.currentIndex;
		if (indexObject > -1)
		{
			if (this.arrayObject[indexObject].isSite)
			{
				if (confirm(this.stringBundle.getString("confirmSiteSettingRemove")))
				{
					// Delete the site setting directory
					var siteSettingDir = new this.nsILocalFile();
					siteSettingDir.initWithFile(this.flashSettingDir);
					siteSettingDir.appendRelativePath('#' + this.arrayObject[indexObject].site);
					this.deleteFile(siteSettingDir, true);

					// Update array and user interface
					if (this.arrayObject[indexObject].used === 0)
					{
						this.deleteTreeSelectionSingle(this.treeObject, this.viewObject, this.arrayObject);
					}
					else
					{
						this.arrayObject[indexObject].limit = -10;
						this.arrayObject[indexObject].privacy = -10;
						this.treeObject.treeBoxObject.invalidateRow(indexObject);
						document.getElementById("settingSiteSpace").value = -10;
						document.getElementById("settingSiteCamera").value = -10;
					}
					this.arrayObject[indexObject].file = null;
				}
			}
		}
	},

	deleteSiteAll: function()
	{
		if (confirm(this.stringBundle.getString("confirmSiteRemoveAll")))
		{
			var treeBox = this.treeObject.treeBoxObject;
			treeBox.beginUpdateBatch();
			for (var j=this.arrayObject.length-1; j>-1; j--)
			{
				if (this.arrayObject[j].isSite)
				{
					// Update array
					if (this.arrayObject[j].used === 0)
					{
						this.arrayObject.splice(j, 1);
						this.viewObject.rowCount--;
						treeBox.rowCountChanged(j, -1);
					}
					else
					{
						this.arrayObject[j].limit = -10;
						this.arrayObject[j].privacy = -10;
						this.arrayObject[j].file = null;
					}
				}
			}
			treeBox.view.selection.select(-1);
			treeBox.endUpdateBatch();
			treeBox.ensureRowIsVisible(this.treeObject.currentIndex);

			// Delete all settings directories
			try
			{
				var entries = this.flashSettingDir.directoryEntries;
			} catch(err)
			{
				return;
			}
			while(entries.hasMoreElements())
			{
				var entry = entries.getNext();
				entry.QueryInterface(Components.interfaces.nsIFile);
				if (entry.isDirectory())
				{
					this.deleteFile(entry, true);
				}
			}
		}
	},

	// user interface event: save site settings
	saveSiteSetting: function()
	{
		var indexObject = this.treeObject.currentIndex;
		if (indexObject > -1)
		{
			// Get user values
			var siteLimit = document.getElementById("settingSiteSpace").value;
			var sitePrivacy = document.getElementById("settingSiteCamera").value;

			// Update array and user interface
			this.arrayObject[indexObject].limit = siteLimit;
			this.arrayObject[indexObject].privacy = sitePrivacy;
			this.treeObject.treeBoxObject.invalidateRow(indexObject);

			// Create site setting file object
			var siteSettingDir = new this.nsILocalFile();
			siteSettingDir.initWithFile(this.flashSettingDir);
			siteSettingDir.appendRelativePath('#' + this.arrayObject[indexObject].site);
			var siteSettingFile = new this.nsILocalFile();
			siteSettingFile.initWithFile(siteSettingDir);
			siteSettingFile.append('settings.sol');

			// If settings are set to global then we delete the site setting directory
			if (siteLimit == -10 && sitePrivacy == -10)
			{
				this.deleteFile(siteSettingDir, true);
			}
			else
			{
				if (!siteSettingDir.exists())
				{
					siteSettingDir.create(1,0777);
				}
				// Create a new xml setting document based on user selection
				var xmlSiteSetting = Components.classes["@mozilla.org/xml/xml-document;1"].createInstance(Components.interfaces.nsIDOMDocument);
				var xmlSiteSettingData = this.xmlSettingFile.createElement("data");
				xmlSiteSetting.appendChild(xmlSiteSettingData);
				var xmlName = this.addXmlSetting(xmlSiteSetting, xmlSiteSettingData, 0, 'name', this.arrayObject[indexObject].site + '/settings', '3', 0, false, 0, 0);
				if (siteLimit != -10)
				{
					var eleSiteSpace = this.addXmlSetting(xmlSiteSetting, xmlSiteSettingData, 1, 'number', 'klimit', siteLimit, 0, false, 0, 0);
				}
				if (sitePrivacy != -10)
				{
					var siteSecurityAllow = (sitePrivacy == 2) ? 'true' : 'false';
					var siteSecurityAlways = (sitePrivacy == 2 || sitePrivacy == 3) ? 'true' : 'false';
					eleSiteSecurityAllow = this.addXmlSetting(xmlSiteSetting, xmlSiteSettingData, 2, 'boolean', 'allow', siteSecurityAllow, 0, false, 0, 0);
					eleSiteSecurityAlways = this.addXmlSetting(xmlSiteSetting, xmlSiteSettingData, 3, 'boolean', 'always', siteSecurityAlways, 0, false, 0, 0);
				}
				var writeSuccess = this.nsISolRequest.solWrite(xmlSiteSetting, siteSettingFile);
				if (writeSuccess)
				{
					this.jsdump('OBJECTION-Content saveSiteSetting:success: ' + siteSettingFile.path, 3);
					this.arrayObject[indexObject].file = siteSettingFile;
				}
				else
				{
					this.jsdump('OBJECTION-Content saveSiteSetting:failed: ' + siteSettingFile.path, 5);
				}
			}
		}
	},

	// user interface event: save global settings
	saveGlobalSetting: function()
	{
		this.updateGlobalSetting();
		var writeSuccess = this.writeGlobalSetting(false);
	},

	updateGlobalSetting: function()
	{
		// Read the settings from the user interface
		var settingGlobalCamera = document.getElementById("settingGlobalCamera").value;
		var settingGlobalSpace = document.getElementById("settingGlobalSpace").value;
		var settingGlobalThird = (document.getElementById("settingGlobalThird").checked === true) ? 'true' : 'false';
		var settingGlobalCache = (document.getElementById("settingGlobalCache").checked === true) ? '-1' : '0';
		var settingGlobalSecurity = document.getElementById("settingGlobalSecurity").value;
		var settingGlobalSecurityAllow = (settingGlobalSecurity == 2) ? 'true' : 'false';
		var settingGlobalSecurityAlways = (settingGlobalSecurity == 2 || settingGlobalSecurity == 3) ? 'true' : 'false';
		var settingGlobalUpdate = document.getElementById("settingGlobalUpdate").value;
		var settingGlobalUpdateDisabled = (settingGlobalUpdate == '-1') ? 'true' : 'false';
		var settingGlobalUpdateInterval = (settingGlobalUpdate != '-1') ? settingGlobalUpdate : '30';
		var settingVideoAcceleration = (document.getElementById("settingVideoAcceleration").checked == 'true') ? 'true' : 'false';
		// If the settings xml Document does not exist, create it
		if (this.xmlSettingFile === null)
		{
			this.xmlSettingFile = Components.classes["@mozilla.org/xml/xml-document;1"].createInstance(Components.interfaces.nsIDOMDocument);
			this.xmlSettingFileData = this.xmlSettingFile.createElement("data");
			this.xmlSettingFile.appendChild(this.xmlSettingFileData);
			var xmlName = this.addXmlSetting(this.xmlSettingFile, this.xmlSettingFileData, 0, 'name', 'settings', '3', 0, false, 0, 0);
		}
		// Update/add the settings in the xml Document
		this.eleGlobalCamera = this.updateXmlSetting(this.xmlSettingFile, this.xmlSettingFileData, this.eleGlobalCamera, 'boolean', 'defaultalways', settingGlobalCamera);
		this.eleGlobalSpace = this.updateXmlSetting(this.xmlSettingFile, this.xmlSettingFileData, this.eleGlobalSpace, 'number', 'defaultklimit', settingGlobalSpace);
		this.eleGlobalThird = this.updateXmlSetting(this.xmlSettingFile, this.xmlSettingFileData, this.eleGlobalThird, 'boolean', 'allowThirdPartyLSOAccess', settingGlobalThird);
		this.eleGlobalCache = this.updateXmlSetting(this.xmlSettingFile, this.xmlSettingFileData, this.eleGlobalCache, 'number', 'secureCrossDomainCacheSize', settingGlobalCache);
		this.eleGlobalSecurityAllow = this.updateXmlSetting(this.xmlSettingFile, this.xmlSettingFileData, this.eleGlobalSecurityAllow, 'boolean', 'crossdomainAllow', settingGlobalSecurityAllow);
		this.eleGlobalSecurityAlways = this.updateXmlSetting(this.xmlSettingFile, this.xmlSettingFileData, this.eleGlobalSecurityAlways, 'boolean', 'crossdomainAlways', settingGlobalSecurityAlways);
		this.eleGlobalUpdateDisabled = this.updateXmlSetting(this.xmlSettingFile, this.xmlSettingFileData, this.eleGlobalUpdateDisabled, 'boolean', 'autoUpdateDisabled', settingGlobalUpdateDisabled);
		this.eleGlobalUpdateInterval = this.updateXmlSetting(this.xmlSettingFile, this.xmlSettingFileData, this.eleGlobalUpdateInterval, 'number', 'autoUpdateInterval', settingGlobalUpdateInterval);
		this.eleGlobalVideoAcceleration = this.updateXmlSetting(this.xmlSettingFile, this.xmlSettingFileData, this.eleGlobalVideoAcceleration, 'boolean', 'safefullscreen', settingVideoAcceleration);
	},

	writeGlobalSetting: function(isSilent)
	{
		var writeSuccess = false;
		// If the Flash Player folder exists
		if (this.flashAppDir.path !== null && this.xmlSettingFile !== null)
		{
			// If the Flash Player settings folder does not exist, create it
			if (this.flashSettingDir === null)
			{
				this.flashSettingDir.initWithFile(this.flashAppDir);
				this.flashSettingDir.appendRelativePath('macromedia.com');
				this.flashSettingDir.appendRelativePath('support');
				this.flashSettingDir.appendRelativePath('flashplayer');
				this.flashSettingDir.appendRelativePath('sys');
				if (!siteSettingDir.exists())
				{
					siteSettingDir.create(1,0777);
				}
			}
			// If the Flash Player settings file does not exist, create it
			if (this.flashSettingFile === null)
			{
				this.flashSettingFile = new this.nsILocalFile();
				this.flashSettingFile.initWithFile(this.flashSettingDir);
				this.flashSettingFile.append('settings.sol');
			}
			// Create a copy of the setting xml document to add the domains and third party sites to
			var flashSettingFileCopy = Components.classes["@mozilla.org/xml/xml-document;1"].createInstance(Components.interfaces.nsIDOMDocument);
			var flashSettingFileCopyData = flashSettingFileCopy.importNode(this.xmlSettingFileData,true);
			flashSettingFileCopy.appendChild(flashSettingFileCopyData);
			this.addSettingFileSite(flashSettingFileCopy, flashSettingFileCopyData);
			this.addSettingFileThird(flashSettingFileCopy, flashSettingFileCopyData);
			writeSuccess = this.nsISolRequest.solWrite(flashSettingFileCopy, this.flashSettingFile);
			if (!isSilent)
			{
				if (writeSuccess)
				{
					alert(this.stringBundle.getString("messageSettingSaveGlobalSuccess"));
				}
				else
				{
					alert(this.stringBundle.getString("messageSettingSaveGlobalFailure"));
					this.flashSettingFile = null;
				}
			}
		}
		return writeSuccess
	},

	updateXmlSetting: function(xmlDoc, xmlData, xmlElement, settingType, settingName, settingValue)
	{
		var returnXmlElement = null;
		if (xmlElement !== null)
		{
			var xmlValueNew = xmlDoc.createTextNode(settingValue);
			xmlElement.replaceChild(xmlValueNew, xmlElement.firstChild);
			returnXmlElement = xmlElement;
		}
		else
		{
			returnXmlElement = this.addXmlSetting(xmlDoc, xmlData, 999, settingType, settingName, settingValue, 0, false, 0, 0);
		}
		return returnXmlElement;
	},

	addXmlSetting: function(xmlDoc, xmlData, numberSetting, settingType, settingName, settingValue, settingLevel, settingContainer, settingTarget, settingParent)
	{
		var xmlElementNew = xmlDoc.createElement("element");
		xmlElementNew.setAttribute("number", numberSetting);
		xmlElementNew.setAttribute("name", settingName);
		xmlElementNew.setAttribute("type", settingType);
		xmlElementNew.setAttribute("level", settingLevel);
		xmlElementNew.setAttribute("container", settingContainer);
		xmlElementNew.setAttribute("target", settingTarget);
		xmlElementNew.setAttribute("parent", settingParent);
		var xmlValueNew = xmlDoc.createTextNode(settingValue);
		xmlElementNew.appendChild(xmlValueNew);
		xmlData.appendChild(xmlElementNew);
		return xmlElementNew;
	},

	addSettingFileSite: function(xmlDoc, xmlData)
	{
		// Add the domains object
		var tempElement = null;
		tempElement = this.addXmlSetting(xmlDoc, xmlData, 1000, 'object', 'domains', '', 0, true, 1000, 0);
		// Add the domains
		for (var i=0; i < this.arrayObject.length; i++)
		{
			var fileExists = (this.arrayObject[i].file === null) ? false : this.arrayObject[i].file.exists();
			if (this.arrayObject[i].isSite && (this.arrayObject[i].used > 0 || fileExists))
			{
				var siteNew = (this.arrayObject[i].site == 'localhost') ? 'local' : this.arrayObject[i].site;
				tempElement = this.addXmlSetting(xmlDoc, xmlData, 1000 + i, 'boolean', siteNew, 'true', 1, false, 0, 1000);
			}
		}
	},

	addSettingFileThird: function(xmlDoc, xmlData)
	{
		// Add the trustedPaths object
		var tempElement = null;
		tempElement = this.addXmlSetting(xmlDoc, xmlData, 2000, 'object', 'trustedPaths', '', 0, true, 2000, 0);
		// Add the trustedPaths
		for (var i=0; i < this.arraySettingThird.length; i++)
		{
			tempElement = this.addXmlSetting(xmlDoc, xmlData, 2000 + i, 'boolean', this.arraySettingThird[i].name, 'true', 1, false, 0, 2000);
		}
	},

	deleteThirdSelected: function()
	{
		var indexObject = this.treeSettingThird.currentIndex;
		if (indexObject > -1)
		{
			if (confirm(this.stringBundle.getString("confirmThirdRemove")))
			{
				// Delete from tree and update (Changes will be saved on close)
				this.deleteTreeSelectionSingle(this.treeSettingThird, this.viewSettingThird, this.arraySettingThird);
				this.treeSettingThirdSelected();
			}
		}
	},

	deleteThirdAll: function()
	{
		if (confirm(this.stringBundle.getString("confirmThirdRemoveAll")))
		{
			// Delete from tree and update (Changes will be saved on close)
			var treeBox = this.treeSettingThird.treeBoxObject;
			treeBox.beginUpdateBatch();
			this.viewSettingThird.rowCount = 0;
			treeBox.rowCountChanged(0, -this.arraySettingThird.length);
			treeBox.view.selection.select(-1);
			this.arraySettingThird = new Array();
			treeBox.endUpdateBatch();
			this.deleteTreeSelectionSingle(this.treeSettingThird, this.viewSettingThird, this.arraySettingThird);
			this.treeSettingThirdSelected();
		}
	},

	closeSave: function()
	{
		this.closeGeneric();
	},

	closeCancel: function()
	{
		this.closeGeneric();
	},

	closeGeneric: function()
	{
		try
		{
			this.deleteUnusedDir();
			if (this.flashSettingFile !== null)
			{
				var writeSuccess = this.writeGlobalSetting(true);
				if (writeSuccess)
				{
					this.jsdump('OBJECTION-Content closeGeneric:success: ' + this.flashSettingFile.path, 3);
				}
				else
				{
					this.jsdump('OBJECTION-Content closeGeneric:failed: ' + this.flashSettingFile.path, 5);
				}
			}
			else
			{
				this.jsdump('OBJECTION-Content closeGeneric:failed:No Flash Setting File', 3);
			}
		}
		catch (err)
		{
			this.jsdump('OBJECTION-Content closeGeneric:err: ' + err, 5);
		}
	},

	showSettingFlash: function()
	{
		document.getElementById("settingGlobalCamera").value = (this.eleGlobalCamera !== null) ? this.eleGlobalCamera.firstChild.nodeValue : "false";
		document.getElementById("settingGlobalSpace").value = (this.eleGlobalSpace !== null) ? this.eleGlobalSpace.firstChild.nodeValue : '100';
		document.getElementById("settingGlobalThird").checked = (this.eleGlobalThird !== null) ? this.toBoolean(this.eleGlobalThird.firstChild.nodeValue) : false;
		document.getElementById("settingGlobalCache").checked = (this.eleGlobalCache !== null) ? this.toBoolean(this.eleGlobalCache.firstChild.nodeValue) : false;
		var bolGlobalSecurityAllow = (this.eleGlobalSecurityAllow !== null) ? this.toBoolean(this.eleGlobalSecurityAllow.firstChild.nodeValue) : false;
		var bolGlobalSecurityAlways = (this.eleGlobalSecurityAlways !== null) ? this.toBoolean(this.eleGlobalSecurityAlways.firstChild.nodeValue) : false;
		document.getElementById("settingGlobalSecurity").value = (bolGlobalSecurityAllow && bolGlobalSecurityAlways) ? '2' : (!bolGlobalSecurityAllow && bolGlobalSecurityAlways) ? '3' : '1';
		var bolGlobalUpdateDisabled = (this.eleGlobalUpdateDisabled !== null) ? this.toBoolean(this.eleGlobalUpdateDisabled.firstChild.nodeValue) : true;
		var valGlobalUpdateInterval = (this.eleGlobalUpdateInterval !== null) ? this.eleGlobalUpdateInterval.firstChild.nodeValue : '30';
		document.getElementById("settingGlobalUpdate").value = (bolGlobalUpdateDisabled) ? '-1' : valGlobalUpdateInterval;
		document.getElementById("settingVideoAcceleration").checked = (this.eleGlobalVideoAcceleration !== null) ? !this.toBoolean(this.eleGlobalVideoAcceleration.firstChild.nodeValue) : true;
	},

	toBoolean: function(valueString)
	{
		var returnBoolean = false;
		if (valueString === null || valueString === undefined)
		{
			returnBoolean = false;
		}
		else
		{
			testString = valueString.toString();
			if (testString.toLowerCase() == 'true')
			{
				returnBoolean = true;
			}
			else if (testString.toLowerCase() == 'false')
			{
				returnBoolean = false;
			}
			else
			{
				returnBoolean = Boolean(parseInt(valueString));
			}
		}
		return returnBoolean;
	},

	deleteUnusedDir: function()
	{
		var existsSetting = (this.flashSettingDir === null) ? false : true;
		var existsObject = (this.flashObjectDir === null) ? false : true;
		var settingDir = new this.nsILocalFile();
		var objectDir = new this.nsILocalFile();
		for (var i=0; i < this.arrayObject.length; i++)
		{
			if (this.arrayObject[i].isSite)
			{
				if (this.arrayObject[i].file === null && existsSetting)
				{
					settingDir.initWithFile(this.flashSettingDir);
					settingDir.appendRelativePath('#' + this.arrayObject[i].site);
					if (settingDir.exists() && settingDir.isDirectory())
					{
						this.deleteFile(settingDir, true);
					}
				}
				if (this.arrayObject[i].used === 0 && existsObject)
				{
					objectDir.initWithFile(this.flashObjectDir);
					objectDir.appendRelativePath(this.arrayObject[i].site);
					if (objectDir.exists() && objectDir.isDirectory())
					{
						this.deleteFile(objectDir, true);
					}
				}
			}
		}
	},

	resetTree: function(tree, view, table)
	{
			var treeBox = tree.treeBoxObject;
			treeBox.beginUpdateBatch();
			view.rowCount = 0;
			treeBox.rowCountChanged(0, -table.length);
			treeBox.view.selection.select(-1);
			table = new Array();
			treeBox.endUpdateBatch();
	},

	onAutomaticRemoveChange: function()
	{
		if (document.getElementById("automaticRemove").value == 'never')
		{
			document.getElementById("automaticSite").setAttribute("disabled", "true");
		}
		else
		{
			document.getElementById("automaticSite").removeAttribute("disabled");
		}
	},

	// user interface event: change directory
	changeAppDir: function()
	{
		var filePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance(this.nsIFilePicker);
		filePicker.init(window, this.stringBundle.getString("selectDirTitle"), this.nsIFilePicker.modeGetFolder);
		var rv = filePicker.show();
		if (rv == this.nsIFilePicker.returnOK)
		{
			var file = filePicker.file;
			//this.setAppDir(file.path);
			if (document.getElementById("flashAppDir").value != file.path)
			{
				document.getElementById("flashAppDir").value = file.path;
				this.prefPlatform.setCharPref(this.thisPlatform, file.path);
				this.flashAppDir.initWithFile(file);
				// Reset paths and trees
				this.resetPathsAndTrees();
				this.main();
			}
		}
	},

	// user interface event: automatic search
	searchAppDir: function()
	{
		this.flashAppDir = this.getFlashAppDir();
		// If Flash data directory could not be found then show error
		if (this.flashAppDir === null)
		{
			document.getElementById("flashAppDir").value = this.stringBundle.getString("dirNotFound");
		}
		else
		{
			//this.setAppDir(this.flashAppDir.path);
			// Only run main if the directory has changed
			if (document.getElementById("flashAppDir").value != this.flashAppDir.path)
			{
				document.getElementById("flashAppDir").value = this.flashAppDir.path;
				this.prefPlatform.setCharPref(this.thisPlatform, this.flashAppDir.path);
				// Reset paths and trees
				this.resetPathsAndTrees();
				this.main();
			}
		}
	},

	// Reset paths and trees (called after a change in flashAppDir)
	resetPathsAndTrees: function()
	{
		this.flashSettingDir = new this.nsILocalFile();
		this.flashSettingFile = new this.nsILocalFile();
		this.flashObjectDir = new this.nsILocalFile();
		this.countObject = 0;
		this.countObjectDetail = 0;
		this.countSettingThird = 0;
		this.resetTree(this.treeObject, this.viewObject, this.arrayObject);
		this.resetTree(this.treeObjectDetail, this.viewObjectDetail, this.eleObjectDetail);
		this.resetTree(this.treeSettingThird, this.viewSettingThird, this.arraySettingThird);
	},

	// Automatic detection of Flash Apps directory
	getFlashAppDir: function()
	{
		var macromediaDir = null;
		var flashDir = null;
		var testDir = null;
		var tryAppData = null;
		try
		{
			tryAppData = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("AppData", Components.interfaces.nsILocalFile);
		} catch(err) {}
		if (tryAppData !== null)
		{
			this.jsdump('OBJECTION-Content getFlashAppDir:found:AppData: ' + tryAppData.path, 1);
			testDir = tryAppData;
		}
		else
		{
			var tryUsrPrfs = null;
			try
			{
				tryUsrPrfs = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("UsrPrfs", Components.interfaces.nsILocalFile);
			} catch(err) {}
			if (tryUsrPrfs !== null)
			{
				this.jsdump('OBJECTION-Content getFlashAppDir:found:UsrPrfs: ' + tryUsrPrfs.path, 1);
				testDir = tryUsrPrfs;
			}
			else
			{
				var tryHome = null;
				try
				{
					tryHome = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("Home", Components.interfaces.nsILocalFile);
				} catch(err) {}
				if (tryHome !== null)
				{
					this.jsdump('OBJECTION-Content getFlashAppDir:found:Home: ' + tryHome.path, 1);
					testDir = tryHome;
				}
			}
		}
		if (testDir !== null)
		{
			testRegExp = /^\.?macromedia$/i;
			macromediaDir = this.findDirectory(testDir, testRegExp);
			if (macromediaDir !== null)
			{
				testRegExp = /^flash[ |_]player$/i;
				flashDir = this.findDirectory(macromediaDir, testRegExp);
				if (flashDir !== null)
				{
					this.prefPlatform.setCharPref(this.thisPlatform, flashDir.path);
					return flashDir;
				}
				else
				{
					this.jsdump('OBJECTION-Content getFlashAppDir:found:Macromedia:(No Flash Player) ' + macromediaDir, 1);
					this.prefPlatform.setCharPref(this.thisPlatform, "");
					return null;
				}
			}
			else
			{
				this.jsdump('OBJECTION-Content getFlashAppDir:No Macromedia:', 1);
				this.prefPlatform.setCharPref(this.thisPlatform, "");
				return null;
			}
		}
		else
		{
			this.jsdump('OBJECTION-Content getFlashAppDir:No user directory:Platform ' + this.thisPlatform, 5);
			this.prefPlatform.setCharPref(this.thisPlatform, "");
			return null;
		}
	},

	findDirectory: function(startDirectory, regDirectory)
	{
		var returnDirectory = null;
		try
		{
			var entries = startDirectory.directoryEntries;
		} catch(err)
		{
			return;
		}
		while(entries.hasMoreElements())
		{
			var entry = entries.getNext();
			entry.QueryInterface(Components.interfaces.nsIFile);
			if (entry.isDirectory())
			{
				this.jsdump('OBJECTION-Content findDirectory:entry: ' + entry.path, 1);
				if (regDirectory.test(entry.leafName))
				{
					this.jsdump('OBJECTION-Content findDirectory:found: ' + entry.path, 3);
					return entry;
				}
				else
				{
					returnDirectory = this.findDirectory(entry, regDirectory);
					if (returnDirectory !== null)
					{
						return returnDirectory;
					}
				}
			}
		}
		return returnDirectory;
	},

	sortArrayAscending: function(arrayTarget, arrayOrder)
	{
		// Create the sort function
		var me = this;
		var compareFunc = function compare(first, second) {return me.compareArray(first, second, arrayOrder);};
		// Do the sort
		arrayTarget.sort(compareFunc);
	},

	compareArray: function(first, second, sortOrder)
	{
		if (sortOrder.length === 0)
		{
			return 0;
		}

		var sortOrderSub = sortOrder.slice(1);
		var column = sortOrder[0];

		if (isNaN(first[column]) || isNaN(second[column]))
		{
			var x = first[column].toLowerCase();
			var y = second[column].toLowerCase();
		}
		else
		{
			var x = first[column];
			var y = second[column];
		}
		return ((x < y) ? -1 : ((x > y) ? 1 : this.compareArray(first, second, sortOrderSub)));
	},

	deleteTreeSelectionSingle: function(tree, view, table)
	{
		var treeBox = tree.treeBoxObject;
		var indexSelection = tree.currentIndex;

		treeBox.beginUpdateBatch();
		view.rowCount--;
		treeBox.rowCountChanged(indexSelection, -1);
		table.splice(indexSelection, 1);
		treeBox.view.selection.select(indexSelection < table.length ? indexSelection : table.length - 1);
		treeBox.endUpdateBatch();
		treeBox.ensureRowIsVisible(tree.currentIndex);
	},

	deleteObjectSiteDir: function(siteName)
	{
		var siteDir = new this.nsILocalFile();
		siteDir.initWithFile(this.flashObjectDir);
		siteDir.appendRelativePath(siteName);
		this.deleteFile(siteDir, true);
	},

	deleteFile: function(file, recursive)
	{
		if (file.exists())
		{
			try
			{
				file.remove(recursive);
				this.jsdump('OBJECTION-Content deleteFile:success: ' + file.path, 3);
			}
			catch (err)
			{
				this.jsdump('OBJECTION-Content deleteFile:failed: ' + file.path + ' err:'+err, 5);
			}
		}
	},

	// Evaluate an XPath expression aExpression against a given DOM node
	// or Document object (aNode), returning the results as an array
	// thanks wanderingstan at morethanwarm dot mail dot com for the
	// initial work.
	// http://developer.mozilla.org/en/docs/Using_XPath
	evaluateXPath: function(aNode, aExpr)
	{
		var xpe = new XPathEvaluator();
		var nsResolver = xpe.createNSResolver(aNode.ownerDocument == null ? aNode.documentElement : aNode.ownerDocument.documentElement);
		var result = xpe.evaluate(aExpr, aNode, nsResolver, 0, null);
		result.QueryInterface(Components.interfaces.nsIDOMXPathResult);  // not necessary in FF3, only 2.x and possibly earlier
		var found = [];
		var res;
		while (res = result.iterateNext())
		{
			found.push(res);
		}
		return found;
	},

	jsdump: function(str, level)
	{
		if (level >= objection.debugLevel)
		{
  			Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService).logStringMessage(str);
		}
	}
};